@@ -1,30 +1,11 @@
 #!/usr/bin/env python
+
 """
+Hacker News API
+Unofficial Python API for Hacker News.
+
 @author Karan Goel
 @email karan@goel.im
-
-The MIT License (MIT)
-Copyright (c) 2013 Karan Goel
-
-Permission is hereby granted, free of charge, to any person
-obtaining a copy of this software and associated documentation
-files (the "Software"), to deal in the Software without restriction,
-including without limitation the rights to use, copy, modify,
-merge, publish, distribute, sublicense, and/or sell copies of the
-Software, and to permit persons to whom the Software is furnished
-to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be
-included in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
-BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
-ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
-CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-SOFTWARE.
 """
 
 
@@ -43,15 +24,14 @@ class HN(object):
     def __init__(self):
         self.more = ''
 
-    def _get_next_page(self, soup):
-        """
-        Get the relative url of the next page (The "More" link at
-        the bottom of the page)
-        """
-        table = soup.findChildren('table')[2] # the table with all submissions
-
-        # the last row of the table contains the relative url of the next page
-        return table.findChildren(['tr'])[-1].find('a').get('href').lstrip('//')
+    # def _get_next_page(self, soup):
+    #     """
+    #     Get the relative url of the next page (The "More" link at
+    #     the bottom of the page)
+    #     """
+    #     table = soup.findChildren('table')[2] # the table with all submissions
+    #     # the last row of the table contains the relative url of the next page
+    #     return table.findChildren(['tr'])[-1].find('a').get('href').replace(BASE_URL, '').lstrip('//')
 
     def _get_zipped_rows(self, soup):
         """
@@ -87,12 +67,12 @@ class HN(object):
 
             is_self = False # by default all stories are linking posts
 
-            if link.find('http') is -1 : # the link doesn't contains "http" meaning an internal link
+            if link.find('item?id=') is -1: # the link doesn't contains "http" meaning an internal link
+                domain = info_cells[2].find('span').string[2:-2] # slice " (abc.com) "
+            else:
                 link = '%s/%s' % (BASE_URL, link)
                 domain = BASE_URL
                 is_self = True
-            else:
-                domain = info_cells[2].find('span').string[2:-2] # slice " (abc.com) "
             #-- Get the into about a story --#
 
             #-- Get the detail about a story --#
@@ -146,22 +126,24 @@ class HN(object):
         of HN.
         'story_type' can be:
         \t'' = top stories (homepage) (default)
+        \t'news2' = page 2 of top stories
         \t'newest' = most recent stories
         \t'best' = best stories
 
-        'limit' is the number of stories required. Defaults to 30
+        'limit' is the number of stories required from the given page.
+        Defaults to 30. Cannot be more than 30.
         """
-        if limit == None or limit < 30:
+        if limit == None or limit < 1 or limit > 30:
             limit = 30 # we need at least 30 items
 
         stories_found = 0
-
+        # self.more = story_type
         # while we still have more stories to find
         while stories_found < limit:
-            soup = get_soup(page=self.more) # get current page soup
+            soup = get_soup(page=story_type) # get current page soup
             all_rows = self._get_zipped_rows(soup)
             stories = self._build_story(all_rows) # get a list of stories on current page
-            self.more = self._get_next_page(soup) # move to next page
+            # self.more = self._get_next_page(soup) # move to next page
 
             for story in stories:
                 yield story
@@ -171,6 +153,21 @@ class HN(object):
                 if stories_found == limit:
                     return
 
+    def get_leaders(self, limit=10):
+        """ Return the leaders of Hacker News """
+        if limit == None:
+            limit = 10
+        soup = get_soup('leaders')
+        table = soup.find('table')
+        leaders_table = table.find_all('table')[1]
+        listLeaders = leaders_table.find_all('tr')[2:]
+        listLeaders.pop(10) # Removing because empty in the Leaders page
+        for i, leader in enumerate(listLeaders):
+            if (i == limit): 
+                return
+            if not leader.text == '':
+                item = leader.find_all('td')
+                yield User(item[1].text,'', item[2].text, item[3].text)
 
 class Story(object):
     """
@@ -199,55 +196,110 @@ class Story(object):
         """
         return '<Story: ID={0}>'.format(self.story_id)
 
+    def _get_next_page(self, soup, current_page):
+        """
+        Get the relative url of the next page (The "More" link at
+        the bottom of the page)
+        """
+
+        # Get the table with all the comments:
+        if current_page == 1:
+            table = soup.findChildren('table')[3]
+        elif current_page > 1:
+            table = soup.findChildren('table')[2]
+
+        # the last row of the table contains the relative url of the next page
+        anchor = table.findChildren(['tr'])[-1].find('a')
+        if anchor and anchor.text == u'More':
+            return anchor.get('href').lstrip(BASE_URL)
+        else:
+            return None
+
     def _build_comments(self, soup):
         """
         For the story, builds and returns a list of Comment objects.
         """
-        
-        table = soup.findChildren('table')[3] # the table holding all comments
-        rows = table.findChildren(['tr']) # get all rows (each comment is duplicated twice)
-        rows = rows[:len(rows) - 2] # last row is more, second last is spacing
-        rows = [row for i, row in enumerate(rows) if (i % 2 == 0)] # now we have unique comments only
 
         comments = []
+        current_page = 1
 
-        if len(rows) > 1:
-            for row in rows:
+        while True:
+            # Get the table holding all comments:
+            if current_page == 1:
+                table = soup.findChildren('table')[3]
+            elif current_page > 1:
+                table = soup.findChildren('table')[2]
 
-                # skip an empty td
-                if not row.findChildren('td'):
-                    continue
+            rows = table.findChildren(['tr']) # get all rows (each comment is duplicated twice)
+            rows = rows[:len(rows) - 2] # last row is more, second last is spacing
+            rows = [row for i, row in enumerate(rows) if (i % 2 == 0)] # now we have unique comments only
 
-                ## Builds a flat list of comments
+            if len(rows) > 1:
+                for row in rows:
 
-                # level of comment, starting with 0
-                level = int(row.findChildren('td')[1].find('img').get('width')) // 40
+                    # skip an empty td
+                    if not row.findChildren('td'):
+                        continue
 
-                spans = row.findChildren('td')[3].findAll('span')
-                # span[0] = submitter details
-                # [<a href="user?id=jonknee">jonknee</a>, u' 1 hour ago  | ', <a href="item?id=6910978">link</a>]
-                # span[1] = actual comment
+                    ## Builds a flat list of comments
 
-                if str(spans[0]) != '<span class="comhead"></span>':
-                    user = spans[0].contents[0].string # user who submitted the comment
-                    time_ago = spans[0].contents[1].string.strip().rstrip(' |') # relative time of comment
-                    comment_id = int(re.match(r'item\?id=(.*)', spans[0].contents[2].get('href')).groups()[0])
+                    # level of comment, starting with 0
+                    level = int(row.findChildren('td')[1].find('img').get('width')) // 40
 
-                    body = spans[1].text # text representation of comment (unformatted)
-                    # html of comment, may not be valid
-                    pat = re.compile(r'<span class="comment"><font color=".*">(.*)</font></span>')
-                    body_html = re.match(pat, str(spans[1]).replace('\n', '')).groups()[0]
-                else:
-                    # comment deleted
-                    user = ''
-                    time_ago = ''
-                    comment_id = -1
-                    body = '[deleted]'
-                    body_html = '[deleted]'
+                    spans = row.findChildren('td')[3].findAll('span')
+                    # span[0] = submitter details
+                    # [<a href="user?id=jonknee">jonknee</a>, u' 1 hour ago  | ', <a href="item?id=6910978">link</a>]
+                    # span[1] = actual comment
 
-                comment = Comment(comment_id, level, user, time_ago, body, body_html)
-                comments.append(comment)
+                    if str(spans[0]) != '<span class="comhead"></span>':
+                        user = spans[0].contents[0].string # user who submitted the comment
+                        time_ago = spans[0].contents[1].string.strip().rstrip(' |') # relative time of comment
+                        try:
+                            comment_id = int(re.match(r'item\?id=(.*)', spans[0].contents[2].get('href')).groups()[0])
+                        except AttributeError:
+                            comment_id = int(re.match(r'%s/item\?id=(.*)' % BASE_URL, spans[0].contents[2].get('href')).groups()[0])
 
+                        body = spans[1].text # text representation of comment (unformatted)
+                        
+                        if body[-2:] == '--':
+                            body = body[:-5]
+
+                        # html of comment, may not be valid
+                        try:
+                            pat = re.compile(r'<span class="comment"><font color=".*">(.*)</font></span>')
+                            body_html = re.match(pat, str(spans[1]).replace('\n', '')).groups()[0]
+                        except AttributeError:
+                            pat = re.compile(r'<span class="comment"><font color=".*">(.*)</font></p><p><font size="1">')
+                            body_html = re.match(pat, str(spans[1]).replace('\n', '')).groups()[0]
+
+                    else:
+                        # comment deleted
+                        user = ''
+                        time_ago = ''
+                        comment_id = -1
+                        body = '[deleted]'
+                        body_html = '[deleted]'
+
+                    comment = Comment(comment_id, level, user, time_ago, body, body_html)
+                    comments.append(comment)
+
+            # Move on to the next page of comments, or exit the loop if there
+            # is no next page.
+            next_page_url = self._get_next_page(soup, current_page)
+            if not next_page_url:
+                break
+
+            soup = get_soup(page=next_page_url)
+            current_page += 1
+
+        # previous_comment = None
+        # for comment in comments:
+        #     if comment.level == 0:
+        #         previous_comment = comment
+        #     else:
+        #         level_difference = comment.level - previous_comment.level
+        #         previous_comment.body_html += '\n' + '\t' * level_difference + comment.body_html
+        #         previous_comment.body += '\n' + '\t' * level_difference + comment.body         
         return comments
     
     @classmethod
@@ -327,3 +379,18 @@ class Comment(object):
         A string representation of the class object
         """
         return '<Comment: ID={0}>'.format(self.comment_id)
+
+class User(object):
+    """
+    Represents a User on HN
+    """
+
+    def __init__(self, username, date_created,karma, avg):
+        self.username = username
+        self.date_created = date_created
+        self.karma = karma
+        self.avg = avg
+
+    def __repr__(self):
+        return '{0} {1} {2}'.format(self.username, self.karma, self.avg)
+
