@@ -1,22 +1,31 @@
-#!/usr/bin/env python
-
 import unittest
+from os import path
 import sys
 
 from hn import HN, Story
+from hn import utils, constants
+
+from test_utils import get_content, PRESETS_DIR
+
+import httpretty
 
 class TestStory(unittest.TestCase):
     
     def setUp(self):
+        httpretty.HTTPretty.enable()
+        httpretty.register_uri(httpretty.GET, '%s/%s' % (constants.BASE_URL, 'item?id=6115341'), 
+            body=get_content('6115341.html'))
+
         self.PY2 = sys.version_info[0] == 2
         if not self.PY2:
             self.text_type = [str]
         else:
             self.text_type = [unicode, str]
-        self.story = Story.fromid(6115341) # https://news.ycombinator.com/item?id=6115341
+        # https://news.ycombinator.com/item?id=6115341
+        self.story = Story.fromid(6115341)
     
     def tearDown(self):
-        pass
+        httpretty.HTTPretty.disable()
     
     def test_story_data_types(self):
         """
@@ -40,6 +49,7 @@ class TestStory(unittest.TestCase):
         Tests the author name
         """
         self.assertEqual(self.story.submitter, 'karangoeluw')
-        
+
+
 if __name__ == '__main__':
-    unittest.main()
\ No newline at end of file
+    unittest.main()
